package ever.generic;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;

import ever.pipeline.AbstractLocalInputAnalyzer;
import ever.pipeline.Case;
import ever.pipeline.CaseFrame;
import ever.pipeline.Clause;
import ever.pipeline.Sentence;
import ever.workflowRepresentation.EmptyProductException;
import ever.workflowRepresentation.NotAProductException;
import ever.workflowRepresentation.Option;
import ever.workflowRepresentation.Product;
import ever.workflowRepresentation.Sequence;
import ever.workflowRepresentation.SimpleProduct;
import ever.workflowRepresentation.Task;
import ever.workflowRepresentation.Workflow;
import ever.workflowRepresentation.WorkflowElement;
import ever.workflowRepresentation.Xor;
/**
 * This analyzer looks for occurences of products which are seperated by an "or". When detecting such cases, an Xor is produced with a path for each product.
 * @author Pol Schumacher, Wirtschaftsinformatik, Institut fuer Informatik, Goethe Universitaet Frankfurt
 *
 */
public class ProductXor extends AbstractLocalInputAnalyzer {

	private String raw;
	private ArrayList<CaseFrame> caseFrameList;
	private ArrayList<Clause> clauseList;
	private Sentence s;
	private Case cas;

	@Override
	public Case analyze(Case cas) {
		this.cas = cas;
		Workflow wf = cas.getWf();
		Sequence seq = null;
		ArrayList<WorkflowElement> taskList = new ArrayList<WorkflowElement>();
		if (wf == null) {
			wf = new Workflow();
			seq = new Sequence(null,null);
		} else {
			seq = (Sequence) wf.getTopSequence();
		}
		Xor xor=null;
		int pos=-1;
		for(WorkflowElement wfe : wf.getTopSequence().getAllElements())
		{
			Task t=null;
			try{
				Task t1=null, t2=null;
				t=(Task)wfe;
				HashSet<Product> pSeen=new HashSet<Product>();
				for(Product p : t.getAllInProducts())
				{
					if(p.getName().contains(" or "))
					{
						String[] split =p.getName().split(" or ");
						t1=t.copy();
						t2=t.copy();
						t1.getAllInProducts().clear();						
						t1.addInProduct(new SimpleProduct(split[0],t.getId()));
						t2.getAllInProducts().clear();
						t2.addInProduct(new SimpleProduct(split[1],t.getId()));						
					}
					else
					{
						pSeen.add(p);
					}
						
				}
				if(t1!=null & t2!=null)
				{
					for(Product p : pSeen)
					{
						t1.addInProduct(p);
						t2.addInProduct(p);
					}
					xor = new Xor(seq, t.relatedSentence);
					Option op1 =new Option(seq, "");
					Sequence seq1=new Sequence(xor, t.relatedSentence);
					seq1.addWfElement(t1);
					op1.setSeq(seq1);
					xor.addOption(op1);
					
					Option op2 =new Option(seq, "");
					Sequence seq2=new Sequence(xor, t.relatedSentence);
					seq2.addWfElement(t2);
					op2.setSeq(seq2);
					xor.addOption(op2);
					pos=seq.getAllElements().indexOf(t);										
				}
			}catch (ClassCastException ex)
			{
				continue;
			} catch (EmptyProductException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (NotAProductException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}															

		}
		
		if(xor!=null)
		{
			seq.getAllElements().remove(pos);
			seq.putElementAt(xor, pos);
			
		}

		wf.setTopSequence(seq);
		cas.setWf(wf);
		return cas;
	}

	@Override
	public HashMap<Integer, Case> analyze(HashMap<Integer, Case> c) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public void analyze() {
		// TODO Auto-generated method stub
		
	}

	
}
